@@ -112,11 +112,7 @@ nsRange::CompareNodeToRange(nsINode* aNode, nsIRange* aRange,
   // if (RANGE(start) <= NODE(start))  and (RANGE(end) => NODE(end))
   // then the Node is contained (completely) by the Range.
   
-  nsresult rv;
-  nsCOMPtr<nsIRange> range = do_QueryInterface(aRange, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  if (!range->IsPositioned()) 
+  if (!aRange || !aRange->IsPositioned()) 
     return NS_ERROR_UNEXPECTED; 
   
   // gather up the dom point info
@@ -135,10 +131,10 @@ nsRange::CompareNodeToRange(nsINode* aNode, nsIRange* aRange,
     nodeEnd = nodeStart + 1;
   }
 
-  nsINode* rangeStartParent = range->GetStartParent();
-  nsINode* rangeEndParent = range->GetEndParent();
-  PRInt32 rangeStartOffset = range->StartOffset();
-  PRInt32 rangeEndOffset = range->EndOffset();
+  nsINode* rangeStartParent = aRange->GetStartParent();
+  nsINode* rangeEndParent = aRange->GetEndParent();
+  PRInt32 rangeStartOffset = aRange->StartOffset();
+  PRInt32 rangeEndOffset = aRange->EndOffset();
 
   // is RANGE(start) <= NODE(start) ?
   PRBool disconnected = PR_FALSE;
@@ -321,8 +317,10 @@ nsRange::ContentRemoved(nsIDocument* aDocument,
   nsINode* container = NODE_FROM(aContainer, aDocument);
 
   // Adjust position if a sibling was removed...
-  if (container == mStartParent && aIndexInContainer < mStartOffset) {
-    --mStartOffset;
+  if (container == mStartParent) {
+    if (aIndexInContainer < mStartOffset) {
+      --mStartOffset;
+    }
   }
   // ...or gravitate if an ancestor was removed.
   else if (nsContentUtils::ContentIsDescendantOf(mStartParent, aChild)) {
@@ -331,8 +329,10 @@ nsRange::ContentRemoved(nsIDocument* aDocument,
   }
 
   // Do same thing for end boundry.
-  if (container == mEndParent && aIndexInContainer < mEndOffset) {
-    --mEndOffset;
+  if (container == mEndParent) {
+    if (aIndexInContainer < mEndOffset) {
+      --mEndOffset;
+    }
   }
   else if (nsContentUtils::ContentIsDescendantOf(mEndParent, aChild)) {
     mEndParent = container;
@@ -411,7 +411,7 @@ nsRange::ComparePoint(nsIDOMNode* aParent, PRInt32 aOffset, PRInt16* aResult)
  ******************************************************/
 
 // Get the length of aNode
-static PRInt32 GetNodeLength(nsINode *aNode)
+static PRUint32 GetNodeLength(nsINode *aNode)
 {
   if(aNode->IsNodeOfType(nsINode::eDATA_NODE)) {
     return static_cast<nsIContent*>(aNode)->TextLength();
@@ -492,7 +492,7 @@ IndexOf(nsIDOMNode* aChildNode)
  ******************************************************/
 
 nsINode*
-nsRange::GetCommonAncestor()
+nsRange::GetCommonAncestor() const
 {
   return mIsPositioned ?
     nsContentUtils::GetCommonAncestor(mStartParent, mEndParent) :
@@ -625,24 +625,29 @@ nsresult nsRange::SetStart(nsIDOMNode* aParent, PRInt32 aOffset)
   VALIDATE_ACCESS(aParent);
 
   nsCOMPtr<nsINode> parent = do_QueryInterface(aParent);
-  nsINode* newRoot = IsValidBoundary(parent);
+  return SetStart(parent, aOffset);
+}
+
+nsresult nsRange::SetStart(nsINode* aParent, PRInt32 aOffset)
+{
+  nsINode* newRoot = IsValidBoundary(aParent);
   NS_ENSURE_TRUE(newRoot, NS_ERROR_DOM_RANGE_INVALID_NODE_TYPE_ERR);
 
-  PRInt32 len = GetNodeLength(parent);
+  PRInt32 len = GetNodeLength(aParent);
   if (aOffset < 0 || aOffset > len)
     return NS_ERROR_DOM_INDEX_SIZE_ERR;
 
   // Collapse if not positioned yet, if positioned in another doc or
   // if the new start is after end.
   if (!mIsPositioned || newRoot != mRoot ||
-      nsContentUtils::ComparePoints(parent, aOffset,
+      nsContentUtils::ComparePoints(aParent, aOffset,
                                     mEndParent, mEndOffset) == 1) {
-    DoSetRange(parent, aOffset, parent, aOffset, newRoot);
+    DoSetRange(aParent, aOffset, aParent, aOffset, newRoot);
 
     return NS_OK;
   }
 
-  DoSetRange(parent, aOffset, mEndParent, mEndOffset, mRoot);
+  DoSetRange(aParent, aOffset, mEndParent, mEndOffset, mRoot);
   
   return NS_OK;
 }
@@ -678,10 +683,16 @@ nsresult nsRange::SetEnd(nsIDOMNode* aParent, PRInt32 aOffset)
   VALIDATE_ACCESS(aParent);
 
   nsCOMPtr<nsINode> parent = do_QueryInterface(aParent);
-  nsINode* newRoot = IsValidBoundary(parent);
+  return SetEnd(parent, aOffset);
+}
+
+
+nsresult nsRange::SetEnd(nsINode* aParent, PRInt32 aOffset)
+{
+  nsINode* newRoot = IsValidBoundary(aParent);
   NS_ENSURE_TRUE(newRoot, NS_ERROR_DOM_RANGE_INVALID_NODE_TYPE_ERR);
 
-  PRInt32 len = GetNodeLength(parent);
+  PRInt32 len = GetNodeLength(aParent);
   if (aOffset < 0 || aOffset > len) {
     return NS_ERROR_DOM_INDEX_SIZE_ERR;
   }
@@ -690,13 +701,13 @@ nsresult nsRange::SetEnd(nsIDOMNode* aParent, PRInt32 aOffset)
   // if the new end is before start.
   if (!mIsPositioned || newRoot != mRoot ||
       nsContentUtils::ComparePoints(mStartParent, mStartOffset,
-                                    parent, aOffset) == 1) {
-    DoSetRange(parent, aOffset, parent, aOffset, newRoot);
+                                    aParent, aOffset) == 1) {
+    DoSetRange(aParent, aOffset, aParent, aOffset, newRoot);
 
     return NS_OK;
   }
 
-  DoSetRange(mStartParent, mStartOffset, parent, aOffset, mRoot);
+  DoSetRange(mStartParent, mStartOffset, aParent, aOffset, mRoot);
 
   return NS_OK;
 }
@@ -1138,24 +1149,16 @@ RemoveNode(nsIDOMNode* aNode)
 }
 
 /**
- * Split a data node into two or three parts.
+ * Split a data node into two parts.
  *
- * @param aStartNode          The original node we are trying to split,
- *                            and first of three.
- * @param aStartIndex         The index at which to split the first and second
- *                            parts.
- * @param aEndIndex           The index at which to split the second and third
- *                            parts.
- * @param aMiddleNode         The second node of three.
- * @param aEndNode            The third node of three.  May be null to indicate
- *                            aEndIndex doesn't apply.
+ * @param aStartNode          The original node we are trying to split.
+ * @param aStartIndex         The index at which to split.
+ * @param aEndNode            The second node.
  * @param aCloneAfterOriginal Set PR_FALSE if the original node should be the
  *                            latter one after split.
  */
 static nsresult SplitDataNode(nsIDOMCharacterData* aStartNode,
                               PRUint32 aStartIndex,
-                              PRUint32 aEndIndex,
-                              nsIDOMCharacterData** aMiddleNode,
                               nsIDOMCharacterData** aEndNode,
                               PRBool aCloneAfterOriginal = PR_TRUE)
 {
@@ -1163,21 +1166,12 @@ static nsresult SplitDataNode(nsIDOMCharacterData* aStartNode,
   nsCOMPtr<nsINode> node = do_QueryInterface(aStartNode);
   NS_ENSURE_STATE(node && node->IsNodeOfType(nsINode::eDATA_NODE));
   nsGenericDOMDataNode* dataNode = static_cast<nsGenericDOMDataNode*>(node.get());
-  // Split the main node, starting with the end.
-  if (aEndNode && aEndIndex > aStartIndex) {
-    nsCOMPtr<nsIContent> newData;
-    rv = dataNode->SplitData(aEndIndex, getter_AddRefs(newData),
-                             aCloneAfterOriginal);
-    NS_ENSURE_SUCCESS(rv, rv);
-    rv = CallQueryInterface(newData, aEndNode);
-    NS_ENSURE_SUCCESS(rv, rv);
-  }
 
   nsCOMPtr<nsIContent> newData;
   rv = dataNode->SplitData(aStartIndex, getter_AddRefs(newData),
                            aCloneAfterOriginal);
   NS_ENSURE_SUCCESS(rv, rv);
-  return CallQueryInterface(newData, aMiddleNode);
+  return CallQueryInterface(newData, aEndNode);
 }
 
 nsresult PrependChild(nsIDOMNode* aParent, nsIDOMNode* aChild)
@@ -1287,13 +1281,20 @@ nsresult nsRange::CutContents(nsIDOMDocumentFragment** aFragment)
 
           if (endOffset > startOffset)
           {
-            nsCOMPtr<nsIDOMCharacterData> cutNode;
-            nsCOMPtr<nsIDOMCharacterData> endNode;
-            rv = SplitDataNode(charData, startOffset, endOffset,
-                               getter_AddRefs(cutNode),
-                               getter_AddRefs(endNode));
+            if (retval) {
+              nsAutoString cutValue;
+              rv = charData->SubstringData(startOffset, endOffset - startOffset,
+                                           cutValue);
+              NS_ENSURE_SUCCESS(rv, rv);
+              nsCOMPtr<nsIDOMNode> clone;
+              rv = charData->CloneNode(PR_FALSE, getter_AddRefs(clone));
+              NS_ENSURE_SUCCESS(rv, rv);
+              clone->SetNodeValue(cutValue);
+              nodeToResult = clone;
+            }
+
+            rv = charData->DeleteData(startOffset, endOffset - startOffset);
             NS_ENSURE_SUCCESS(rv, rv);
-            nodeToResult = cutNode;
           }
 
           handled = PR_TRUE;
@@ -1308,8 +1309,7 @@ nsresult nsRange::CutContents(nsIDOMDocumentFragment** aFragment)
           if (dataLength >= (PRUint32)startOffset)
           {
             nsCOMPtr<nsIDOMCharacterData> cutNode;
-            rv = SplitDataNode(charData, startOffset, dataLength,
-                               getter_AddRefs(cutNode), nsnull);
+            rv = SplitDataNode(charData, startOffset, getter_AddRefs(cutNode));
             NS_ENSURE_SUCCESS(rv, rv);
             nodeToResult = cutNode;
           }
@@ -1327,8 +1327,8 @@ nsresult nsRange::CutContents(nsIDOMDocumentFragment** aFragment)
           /* The Range spec clearly states clones get cut and original nodes
              remain behind, so use PR_FALSE as the last parameter.
           */
-          rv = SplitDataNode(charData, endOffset, endOffset,
-                             getter_AddRefs(cutNode), nsnull, PR_FALSE);
+          rv = SplitDataNode(charData, endOffset, getter_AddRefs(cutNode),
+                             PR_FALSE);
           NS_ENSURE_SUCCESS(rv, rv);
           nodeToResult = cutNode;
         }
@@ -1750,7 +1750,7 @@ nsresult nsRange::CloneContents(nsIDOMDocumentFragment** aReturn)
   return NS_OK;
 }
 
-nsresult nsRange::CloneRange(nsIDOMRange** aReturn)
+nsresult nsRange::DoCloneRange(nsIRange** aReturn) const
 {
   if(mIsDetached)
     return NS_ERROR_DOM_INVALID_STATE_ERR;
@@ -1758,18 +1758,33 @@ nsresult nsRange::CloneRange(nsIDOMRange** aReturn)
   if (aReturn == 0)
     return NS_ERROR_NULL_POINTER;
 
-  nsRange* range = new nsRange();
+  nsRefPtr<nsRange> range = new nsRange();
   NS_ENSURE_TRUE(range, NS_ERROR_OUT_OF_MEMORY);
 
-  NS_ADDREF(*aReturn = range);
-  
   range->SetMaySpanAnonymousSubtrees(mMaySpanAnonymousSubtrees);
 
   range->DoSetRange(mStartParent, mStartOffset, mEndParent, mEndOffset, mRoot);
 
+  *aReturn = range.forget().get();
+
   return NS_OK;
 }
 
+NS_IMETHODIMP nsRange::CloneRange(nsIDOMRange** aReturn)
+{
+  nsIRange* clone;
+  nsresult rv = DoCloneRange(&clone);
+  if (NS_SUCCEEDED(rv)) {
+    *aReturn = clone;
+  }
+  return rv;
+}
+
+nsresult nsRange::CloneRange(nsIRange** aReturn) const
+{
+  return DoCloneRange(aReturn);
+}
+
 nsresult nsRange::InsertNode(nsIDOMNode* aN)
 {
   VALIDATE_ACCESS(aN);
@@ -1948,7 +1963,7 @@ nsresult nsRange::ToString(nsAString& aReturn)
   nsCOMPtr<nsIContentIterator> iter;
   nsresult rv = NS_NewContentIterator(getter_AddRefs(iter));
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = iter->Init(this);
+  rv = iter->Init(static_cast<nsIRange*>(this));
   NS_ENSURE_SUCCESS(rv, rv);
   
   nsString tempString;
