@@ -16,10 +16,11 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2006
+ * Portions created by the Initial Developer are Copyright (C) 2005
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s):
+ * Contributor(s): Igor Bukanov <igor@mir2.org>
+ *                 Bob Clary <bob@bclary.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,30 +36,46 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-360969-04.js';
+var gTestfile = 'regress-230216-1.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 360969;
-var summary = '2^17: global const';
-var actual = 'No Crash';
-var expect = 'No Crash';
-
-var global = this;
+var BUGNUMBER = 230216;
+var summary = 'check for numerical overflow in regexps in back reference and bounds for {} quantifier';
+var actual = '';
+var expect = '';
+var status = '';
 
 printBugNumber(BUGNUMBER);
 printStatus (summary);
- 
-var start = new Date();
-var p;
-var i;
-var limit = 2 << 16;
 
-for (var i = 0; i < limit; i++)
+status = inSection(1) + ' check for overflow in backref';
+
+actual = 'undefined'; 
+expect = false;
+
+try
 {
-  eval('const pv' + i + ';');
+  actual = eval('/(a)\21474836481/.test("aa")');
+}
+catch(e)
+{
+  status += ' Error: ' + e;
 }
 
-reportCompare(expect, actual, summary);
+reportCompare(expect, actual, status);
 
-var stop = new Date();
+status = inSection(2) + ' check for overflow in backref';
+
+actual = 'undefined'; 
+expect = false;
+
+try
+{ 
+  actual = eval('/a\21474836480/.test("")');
+}
+catch(e)
+{
+  status += ' Error: ' + e;
+}
+
+reportCompare(expect, actual, status);
 
-print('Elapsed time: ' + Math.floor((stop - start)/1000) + ' seconds');
