@@ -35,10 +35,10 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-470300-01.js';
+var gTestfile = 'regress-369696-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 470300;
-var summary = 'TM: Do not assert: StackBase(fp) + blockDepth == regs.sp';
+var BUGNUMBER = 396696;
+var summary = 'Do not assert: map->depth > 0" in js_LeaveSharpObject';
 var actual = '';
 var expect = '';
 
@@ -52,12 +52,11 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  jit(true);
 
-  for (let a = 0; a < 3; ++a) { let b = '' + []; }
-
-  jit(false);
+  q = [];
+  q.__defineGetter__("0", q.toString);
+  q[2] = q;
+  q.toSource();
 
   reportCompare(expect, actual, summary);
 
