@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2008
+ * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Gary Kwong
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,86 +35,84 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-452498-102.js';
+var gTestfile = 'regress-351503-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452498;
-var summary = 'TM: upvar2 regression tests';
+var BUGNUMBER = 351503;
+var summary = 'decompilation of TypeError messages';
 var actual = '';
 var expect = '';
 
+printBugNumber(BUGNUMBER);
+printStatus (summary);
 
-//-----------------------------------------------------------------------------
-test();
-//-----------------------------------------------------------------------------
-
-function test()
+expect = 'TypeError: can\'t convert Object to string';
+try
 {
-  enterFunc ('test');
-  printBugNumber(BUGNUMBER);
-  printStatus (summary);
- 
-// ------- Comment #102 From Gary Kwong [:nth10sd]
-
-// =====
-
-  (function(){function x(){} function x()y})();
-
-// Assertion failure: JOF_OPTYPE(op) == JOF_ATOM, at ../jsemit.cpp:1710
-
-// =====
-  function f() {
-    "" + (function(){
-        for( ; [function(){}] ; x = 0)
-          with({x: ""})
-            const x = []
-            });
-  }
-  f();
-
-// Assertion failure: ss->top - saveTop <= 1U, at ../jsopcode.cpp:2156
-
-// =====
-
-  try
-  {
-    function f() {
-      var x;
-      eval("const x = [];");
-    }
-    f();
-  }
-  catch(ex)
-  {
-  }
-// Assertion failure: regs.sp == StackBase(fp), at ../jsinterp.cpp:2984
-
-// =====
-  try
-  {
-    do {x} while([[] for (x in []) ]);
-  }
-  catch(ex)
-  {
-  }
-// Assertion failure: !(pnu->pn_dflags & PND_BOUND), at ../jsemit.cpp:1818
-// =====
-
-  try
-  {
-    {x} ((x=[] for (x in []))); x;
-  }
-  catch(ex)
-  {
-  }
-// Assertion failure: cg->staticLevel >= level, at ../jsemit.cpp:2014
-// Crash [@ BindNameToSlot] in opt without -j
-
-// =====
-
-  reportCompare(expect, actual, summary);
-
-  exitFunc ('test');
+  function p() { return { toString: null } } "fafa".replace(/a/g, p);
+  actual = 'No Error';
 }
+catch(ex)
+{
+  actual = ex + '';
+}
+reportCompare(expect, actual, summary + ': 1');
 
+expect = 'TypeError: can\'t convert Object to string';
+try
+{
+  var a=1; b=2; c={toString: null}; "hahbhc".replace(/[abc]/g, eval);
+  actual = 'No Error';
+}
+catch(ex)
+{
+  actual = ex + '';
+}
+reportCompare(expect, actual, summary + ': 2');
 
+expect = 'TypeError: can\'t convert ({toString:(function () {yield 4;})}) to primitive type';
+try
+{
+  3 + ({toString:function(){yield 4} });
+  actual = 'No Error';
+}
+catch(ex)
+{
+  actual = ex + '';
+}
+reportCompare(expect, actual, summary + ': 3');
 
+expect = 'TypeError: ({}) is not a function';
+try
+{
+  3 + ({toString:({}) }) ;
+  actual = 'No Error';
+}
+catch(ex)
+{
+  actual = ex + '';
+}
+reportCompare(expect, actual, summary + ': 4');
+
+expect = 'TypeError: 3 is not a function';
+try
+{
+  ([5].map)(3);
+  actual = 'No Error';
+}
+catch(ex)
+{
+  actual = ex + '';
+}
+reportCompare(expect, actual, summary + ': 5');
+
+expect = 'TypeError: 5 is not a function';
+try
+{
+  (1 + (4))();
+  actual = 'No Error';
+}
+catch(ex)
+{
+  actual = ex + '';
+}
+reportCompare(expect, actual, summary + ': 6');
