@@ -963,6 +963,8 @@ nsContainerFrame::ReflowOverflowContainerChildren(nsPresContext*           aPres
     return NS_OK; // nothing to reflow
 
   nsOverflowContinuationTracker tracker(aPresContext, this, PR_FALSE, PR_FALSE);
+  PRBool shouldReflowAllKids = aReflowState.ShouldReflowAllKids();
+
   for (nsIFrame* frame = overflowContainers->FirstChild(); frame;
        frame = frame->GetNextSibling()) {
     if (frame->GetPrevInFlow()->GetParent() != GetPrevInFlow()) {
@@ -970,7 +972,9 @@ nsContainerFrame::ReflowOverflowContainerChildren(nsPresContext*           aPres
       // it will get reflowed once it's been placed
       continue;
     }
-    if (NS_SUBTREE_DIRTY(frame)) {
+    // If the available vertical height has changed, we need to reflow
+    // even if the frame isn't dirty.
+    if (shouldReflowAllKids || NS_SUBTREE_DIRTY(frame)) {
       // Get prev-in-flow
       nsIFrame* prevInFlow = frame->GetPrevInFlow();
       NS_ASSERTION(prevInFlow,
@@ -1253,6 +1257,10 @@ nsContainerFrame::SetPropTableFrames(nsPresContext*  aPresContext,
                                      nsIAtom*        aPropID) const
 {
   NS_PRECONDITION(aPresContext && aPropID && aFrameList, "null ptr");
+  NS_PRECONDITION((aPropID != nsGkAtoms::overflowContainersProperty &&
+                   aPropID != nsGkAtoms::excessOverflowContainersProperty) ||
+                  IsFrameOfType(nsIFrame::eCanContainOverflowContainers),
+                  "this type of frame can't have overflow containers");
   nsresult rv = aPresContext->PropertyTable()->SetProperty(this, aPropID,
                   aFrameList, DestroyFrameList, nsnull);
   NS_ASSERTION(rv != NS_PROPTABLE_PROP_OVERWRITTEN, "existing framelist");
