@@ -51,12 +51,14 @@
 #include <shlwapi.h>
 #include <math.h>
 #include <set>
+#include <algorithm>
 #include "resource.h"
 #include "client/windows/sender/crash_report_sender.h"
 #include "common/windows/string_utils-inl.h"
 
 #define CRASH_REPORTER_VALUE L"Enabled"
-#define SUBMIT_REPORT_VALUE  L"SubmitReport"
+#define SUBMIT_REPORT_VALUE  L"SubmitCrashReport"
+#define SUBMIT_REPORT_OLD    L"SubmitReport"
 #define INCLUDE_URL_VALUE    L"IncludeURL"
 #define EMAIL_ME_VALUE       L"EmailMe"
 #define EMAIL_VALUE          L"Email"
@@ -157,10 +159,32 @@ static bool GetBoolValue(HKEY hRegKey, LPCTSTR valueName, DWORD* value)
   return false;
 }
 
+// Removes a value from HKEY_LOCAL_MACHINE and HKEY_CURRENT_USER, if it exists.
+static void RemoveUnusedValues(const wchar_t* key, LPCTSTR valueName)
+{
+  HKEY hRegKey;
+
+  if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, key, 0, KEY_SET_VALUE, &hRegKey) 
+      == ERROR_SUCCESS) {
+    RegDeleteValue(hRegKey, valueName);
+    RegCloseKey(hRegKey);
+  }
+
+  if (RegOpenKeyEx(HKEY_CURRENT_USER, key, 0, KEY_SET_VALUE, &hRegKey) 
+      == ERROR_SUCCESS) {
+    RegDeleteValue(hRegKey, valueName);
+    RegCloseKey(hRegKey);
+  }
+}
+
 static bool CheckBoolKey(const wchar_t* key,
                          const wchar_t* valueName,
                          bool* enabled)
 {
+  /*
+   * NOTE! This code needs to stay in sync with the preference checking
+   *       code in in nsExceptionHandler.cpp.
+   */
   *enabled = false;
   bool found = false;
   HKEY hRegKey;
@@ -188,7 +212,15 @@ static bool CheckBoolKey(const wchar_t* key,
 
 static void SetBoolKey(const wchar_t* key, const wchar_t* value, bool enabled)
 {
+  /*
+   * NOTE! This code needs to stay in sync with the preference setting
+   *       code in in nsExceptionHandler.cpp.
+   */
   HKEY hRegKey;
+
+  // remove the old value from the registry if it exists
+  RemoveUnusedValues(key, SUBMIT_REPORT_OLD);
+
   if (RegCreateKey(HKEY_CURRENT_USER, key, &hRegKey) == ERROR_SUCCESS) {
     DWORD data = (enabled ? 1 : 0);
     RegSetValueEx(hRegKey, value, 0, REG_DWORD, (LPBYTE)&data, sizeof(data));
@@ -1337,20 +1369,45 @@ bool UIGetSettingsPath(const string& vendor,
                        string& settings_path)
 {
   wchar_t path[MAX_PATH];
-  if(SUCCEEDED(SHGetFolderPath(NULL,
-                               CSIDL_APPDATA,
-                               NULL,
-                               0,
-                               path)))  {
-    if (!vendor.empty()) {
-      PathAppend(path, UTF8ToWide(vendor).c_str());
-    }
-    PathAppend(path, UTF8ToWide(product).c_str());
-    PathAppend(path, L"Crash Reports");
-    settings_path = WideToUTF8(path);
-    return true;
+  HRESULT hRes = SHGetFolderPath(NULL,
+                                 CSIDL_APPDATA,
+                                 NULL,
+                                 0,
+                                 path);
+  if (FAILED(hRes)) {
+    // This provides a fallback for getting the path to APPDATA by querying the
+    // registry when the call to SHGetFolderPath is unable to provide this path
+    // (Bug 513958).
+    HKEY key;
+    DWORD type, size, dwRes;
+    dwRes = ::RegOpenKeyExW(HKEY_CURRENT_USER,
+                            L"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders",
+                            0,
+                            KEY_READ,
+                            &key);
+    if (dwRes != ERROR_SUCCESS)
+      return false;
+
+    dwRes = RegQueryValueExW(key,
+                             L"AppData",
+                             NULL,
+                             &type,
+                             (LPBYTE)&path,
+                             &size);
+    ::RegCloseKey(key);
+    // The call to RegQueryValueExW must succeed, the type must be REG_SZ, the
+    // buffer size must not equal 0, and the buffer size be a multiple of 2.
+    if (dwRes != ERROR_SUCCESS || type != REG_SZ || size == 0 || size % 2 != 0)
+        return false;
   }
-  return false;
+
+  if (!vendor.empty()) {
+    PathAppend(path, UTF8ToWide(vendor).c_str());
+  }
+  PathAppend(path, UTF8ToWide(product).c_str());
+  PathAppend(path, L"Crash Reports");
+  settings_path = WideToUTF8(path);
+  return true;
 }
 
 bool UIEnsurePathExists(const string& path)
@@ -1420,3 +1477,46 @@ ofstream* UIOpenWrite(const string& filename, bool append) // append=false
 
   return file;
 }
+
+struct FileData
+{
+  FILETIME timestamp;
+  wstring path;
+};
+
+static bool CompareFDTime(const FileData& fd1, const FileData& fd2)
+{
+  return CompareFileTime(&fd1.timestamp, &fd2.timestamp) > 0;
+}
+
+void UIPruneSavedDumps(const std::string& directory)
+{
+  wstring wdirectory = UTF8ToWide(directory);
+
+  WIN32_FIND_DATA fdata;
+  wstring findpath = wdirectory + L"\\*.dmp";
+  HANDLE dirlist = FindFirstFile(findpath.c_str(), &fdata);
+  if (dirlist == INVALID_HANDLE_VALUE)
+    return;
+
+  vector<FileData> dumpfiles;
+
+  for (BOOL ok = true; ok; ok = FindNextFile(dirlist, &fdata)) {
+    FileData fd = {fdata.ftLastWriteTime, wdirectory + L"\\" + fdata.cFileName};
+    dumpfiles.push_back(fd);
+  }
+
+  sort(dumpfiles.begin(), dumpfiles.end(), CompareFDTime);
+
+  while (dumpfiles.size() > kSaveCount) {
+    // get the path of the oldest file
+    wstring path = (--dumpfiles.end())->path;
+    DeleteFile(path.c_str());
+
+    // s/.dmp/.extra/
+    path.replace(path.size() - 4, 4, L".extra");
+    DeleteFile(path.c_str());
+
+    dumpfiles.pop_back();
+  }
+}
