@@ -59,7 +59,7 @@ nsTSubstring_CharT::nsTSubstring_CharT(const substring_tuple_type& tuple)
 }
 
 nsTSubstring_CharT::nsTSubstring_CharT()
-: mData(const_cast<char_type*>(char_traits::sEmptyBuffer)),
+: mData(char_traits::sEmptyBuffer),
   mLength(0),
   mFlags(F_TERMINATED) {}
 
@@ -101,17 +101,17 @@ nsTSubstring_CharT::MutatePrep( size_type capacity, char_type** oldData, PRUint3
     // able to allocate it.  Just bail out in cases like that.  We don't want
     // to be allocating 2GB+ strings anyway.
     if (capacity > size_type(-1)/2) {
-      // Also assert for |capacity| equal to |size_type(-1)|, since we use that value to
-      // flag immutability.
+      // Also assert for |capacity| equal to |size_type(-1)|, since we used to
+      // use that value to flag immutability.
       NS_ASSERTION(capacity != size_type(-1), "Bogus capacity");
       return PR_FALSE;
     }
 
-    // |curCapacity == size_type(-1)| means that the buffer is immutable, so we
-    // need to allocate a new buffer.  we cannot use the existing buffer even
+    // |curCapacity == 0| means that the buffer is immutable or 0-sized, so we
+    // need to allocate a new buffer. We cannot use the existing buffer even
     // though it might be large enough.
 
-    if (curCapacity != size_type(-1))
+    if (curCapacity != 0)
       {
         if (capacity <= curCapacity) {
           mFlags &= ~F_VOIDED;  // mutation clears voided flag
@@ -272,7 +272,7 @@ nsTSubstring_CharT::ReplacePrep( index_type cutStart, size_type cutLen, size_typ
 nsTSubstring_CharT::size_type
 nsTSubstring_CharT::Capacity() const
   {
-    // return size_type(-1) to indicate an immutable buffer
+    // return 0 to indicate an immutable or 0-sized buffer
 
     size_type capacity;
     if (mFlags & F_SHARED)
@@ -280,9 +280,10 @@ nsTSubstring_CharT::Capacity() const
         // if the string is readonly, then we pretend that it has no capacity.
         nsStringBuffer* hdr = nsStringBuffer::FromData(mData);
         if (hdr->IsReadonly())
-          capacity = size_type(-1);
-        else
+          capacity = 0;
+        else {
           capacity = (hdr->StorageSize() / sizeof(char_type)) - 1;
+        }
       }
     else if (mFlags & F_FIXED)
       {
@@ -298,7 +299,7 @@ nsTSubstring_CharT::Capacity() const
       }
     else
       {
-        capacity = size_type(-1);
+        capacity = 0;
       }
 
     return capacity;
@@ -316,7 +317,7 @@ nsTSubstring_CharT::EnsureMutable( size_type newLen )
 
         // promote to a shared string buffer
         char_type* prevData = mData;
-        Assign(string_type(mData, mLength));
+        Assign(mData, mLength);
         return mData != prevData;
       }
     else
@@ -394,7 +395,12 @@ nsTSubstring_CharT::Assign( const self_type& str )
     if (&str == this)
       return;
 
-    if (str.mFlags & F_SHARED)
+    if (!str.mLength)
+      {
+        Truncate();
+        mFlags |= str.mFlags & F_VOIDED;
+      }
+    else if (str.mFlags & F_SHARED)
       {
         // nice! we can avoid a string copy :-)
 
@@ -410,11 +416,6 @@ nsTSubstring_CharT::Assign( const self_type& str )
         // get an owning reference to the mData
         nsStringBuffer::FromData(mData)->AddRef();
       }
-    else if (str.mFlags & F_VOIDED)
-      {
-        // inherit the F_VOIDED attribute
-        SetIsVoid(PR_TRUE);
-      }
     else
       {
         // else, treat this like an ordinary assignment.
@@ -434,8 +435,17 @@ nsTSubstring_CharT::Assign( const substring_tuple_type& tuple )
 
     size_type length = tuple.Length();
 
-    if (ReplacePrep(0, mLength, length) && length)
+    // don't use ReplacePrep here because it changes the length
+    char_type* oldData;
+    PRUint32 oldFlags;
+    if (MutatePrep(length, &oldData, &oldFlags)) {
+      if (oldData)
+        ::ReleaseData(oldData, oldFlags);
+
       tuple.WriteTo(mData, length);
+      mData[length] = 0;
+      mLength = length;
+    }
   }
 
 void
@@ -545,7 +555,7 @@ nsTSubstring_CharT::Replace( index_type cutStart, size_type cutLength, const sub
       tuple.WriteTo(mData + cutStart, length);
   }
 
-void
+PRBool
 nsTSubstring_CharT::SetCapacity( size_type capacity )
   {
     // capacity does not include room for the terminating null char
@@ -554,7 +564,7 @@ nsTSubstring_CharT::SetCapacity( size_type capacity )
     if (capacity == 0)
       {
         ::ReleaseData(mData, mFlags);
-        mData = const_cast<char_type*>(char_traits::sEmptyBuffer);
+        mData = char_traits::sEmptyBuffer;
         mLength = 0;
         SetDataFlags(F_TERMINATED);
       }
@@ -563,7 +573,7 @@ nsTSubstring_CharT::SetCapacity( size_type capacity )
         char_type* oldData;
         PRUint32 oldFlags;
         if (!MutatePrep(capacity, &oldData, &oldFlags))
-          return; // out-of-memory
+          return PR_FALSE; // out-of-memory
 
         // compute new string length
         size_type newLen = NS_MIN(mLength, capacity);
@@ -585,23 +595,14 @@ nsTSubstring_CharT::SetCapacity( size_type capacity )
         // for backwards compat with the old string implementation.
         mData[capacity] = char_type(0);
       }
+
+    return PR_TRUE;
   }
 
 void
 nsTSubstring_CharT::SetLength( size_type length )
   {
-    if (mLength == length) {
-      mFlags &= ~F_VOIDED;  // mutation clears voided flag
-      return;
-    }
-
-    SetCapacity(length);
-
-    // XXX(darin): SetCapacity may fail, but it doesn't give us a way to find
-    // out.  We should improve that.  For now we just verify that the capacity
-    // changed as expected as a means of error checking.
- 
-    if (Capacity() >= length)
+    if (SetCapacity(length))
       mLength = length;
   }
 
